//
//  RootViewController.m
//  Locations
//
//  Created by NYU User on 5/6/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "RootViewController.h"
#import "Event.h"

@implementation RootViewController
@synthesize eventsArray;
@synthesize managedObjectContext;
@synthesize addButton;
@synthesize locationManager;

#pragma mark -
#pragma mark Initialization

/*
- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    if ((self = [super initWithStyle:style])) {
    }
    return self;
}
*/


#pragma mark -
#pragma mark View lifecycle

- (void) viewDidLoad {
	[super viewDidLoad];
	self.title = @"Locations";
	self.navigationItem.leftBarButtonItem = self.editButtonItem;

	addButton = [[UIBarButtonItem alloc]
		initWithBarButtonSystemItem: UIBarButtonSystemItemAdd
		target: self
		action: @selector(addEvent)
	];

	addButton.enabled = NO;
	self.navigationItem.rightBarButtonItem = addButton;
	[[self locationManager] startUpdatingLocation];

	NSFetchRequest *request = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName: @"Event"
		inManagedObjectContext: managedObjectContext];
	[request setEntity: entity];

	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc]
		initWithKey: @"creationDate" ascending: NO];

	NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:
		sortDescriptor,
		nil
	];
	[request setSortDescriptors: sortDescriptors];
	[sortDescriptors release];
	[sortDescriptor release];
	
	NSError *error = nil;
	NSArray *a =
		[managedObjectContext executeFetchRequest: request error: &error];
	if (a == nil) {
		// Handle the error.
		NSLog(@"executeFetchRequst:error: failed: %@", error);
		[error release];
	}
	
	NSMutableArray *mutableFetchResults = [a mutableCopy];
	
	[self setEventsArray: mutableFetchResults];
	[mutableFetchResults release];
	[request release];
}

/*
- (void) viewWillAppear: (BOOL) animated {
    [super viewWillAppear: animated];
}
*/

/*
- (void) viewDidAppear: (BOOL) animated {
    [super viewDidAppear: animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/
/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (CLLocationManager *) locationManager {
    if (locationManager == nil) {
		locationManager = [[CLLocationManager alloc] init];
		locationManager.desiredAccuracy =
			kCLLocationAccuracyNearestTenMeters;
		locationManager.delegate = self;
	}

    return locationManager;
}

#pragma mark -
#pragma mark Table view data source

- (NSInteger) numberOfSectionsInTableView: (UITableView *) tableView {
    // Return the number of sections.
    return 1;
}


- (NSInteger) tableView: (UITableView *) tableView
  numberOfRowsInSection: (NSInteger) section {

	// Return the number of rows in the section.
	return eventsArray.count;
}


// Customize the appearance of table view cells.
- (UITableViewCell *) tableView: (UITableView *) tableView
	cellForRowAtIndexPath: (NSIndexPath *) indexPath {

	// A date formatter for the time stamp.
	static NSDateFormatter *dateFormatter = nil;
	if (dateFormatter == nil) {
		dateFormatter = [[NSDateFormatter alloc] init];
		[dateFormatter setTimeStyle: NSDateFormatterMediumStyle];
		[dateFormatter setDateStyle: NSDateFormatterMediumStyle];
	}

	// A number formatter for the latitude and longitude.
	static NSNumberFormatter *numberFormatter = nil;
	if (numberFormatter == nil) {
		numberFormatter = [[NSNumberFormatter alloc] init];
		[numberFormatter setNumberStyle: NSNumberFormatterDecimalStyle];
		[numberFormatter setMaximumFractionDigits: 3];
	}

	static NSString *identifier = @"Cell";

	// Dequeue or create a new cell.
	UITableViewCell *cell =
		[tableView dequeueReusableCellWithIdentifier: identifier];
	if (cell == nil) {
		cell = [[UITableViewCell alloc] initWithStyle: UITableViewCellStyleSubtitle
			reuseIdentifier: identifier];
		[cell autorelease];
	}

	Event *event = [eventsArray objectAtIndex: indexPath.row];

	cell.textLabel.text =
		[dateFormatter stringFromDate: event.creationDate];

	cell.detailTextLabel.text = [NSString stringWithFormat: @"%@, %@",
		[numberFormatter stringFromNumber: event.latitude],
		[numberFormatter stringFromNumber: event.longitude]
	];

	return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL) tableView: (UITableView *) tableView canEditRowAtIndexPath: (NSIndexPath *) indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

// support editing the table view.

- (void) tableView: (UITableView *) tableView
	commitEditingStyle: (UITableViewCellEditingStyle) editingStyle
	forRowAtIndexPath: (NSIndexPath *) indexPath {
    
	if (editingStyle == UITableViewCellEditingStyleDelete) {
		// Delete the managed object at the given index path.
        NSManagedObject *eventToDelete = [eventsArray objectAtIndex: indexPath.row];
        [managedObjectContext deleteObject: eventToDelete];
		
        // Update the array and table view.
        [eventsArray removeObjectAtIndex: indexPath.row];
		NSArray *indexPaths = [NSArray arrayWithObject:indexPath];
        [tableView deleteRowsAtIndexPaths:indexPaths withRowAnimation: YES];
		
        // Commit the change.
        NSError *error = nil;
        if (![managedObjectContext save: &error]) {
            // Handle the error.
			NSLog(@"save: failed: %@", error);
			[error release];
        }
    }   
}


/*
// Override to support rearranging the table view.
- (void)tableView: (UITableView *) tableView moveRowAtIndexPath: (NSIndexPath *) fromIndexPath toIndexPath: (NSIndexPath *) toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL) tableView: (UITableView *) tableView canMoveRowAtIndexPath: (NSIndexPath *) indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark -
#pragma mark Table view delegate

- (void) tableView: (UITableView *) tableView
	didSelectRowAtIndexPath: (NSIndexPath *) indexPath {

	// Navigation logic may go here. Create and push another view controller.
	/*
	 <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName: @"<#Nib name#>" bundle: nil];
	 // ...
	 // Pass the selected object to the new view controller.
	 [self.navigationController pushViewController: detailViewController animated: YES];
	 [detailViewController release];
	 */
}

#pragma mark -
#pragma mark Memory management

- (void) didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
	[super didReceiveMemoryWarning];

	// Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void) viewDidUnload {
	// Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
	// For example: self.myOutlet = nil;
	self.eventsArray = nil;
    self.locationManager = nil;
    self.addButton = nil;
}


- (void) dealloc {
	[managedObjectContext release];
	[eventsArray release];
	[locationManager release];
	[addButton release];
	[super dealloc];
}

#pragma mark -
#pragma mark Protocol CLLocationManagerDelegate

- (void) locationManager: (CLLocationManager *) manager
	didUpdateToLocation: (CLLocation *) newLocation
	fromLocation: (CLLocation *) oldLocation {

	addButton.enabled = YES;
}

- (void) locationManager: (CLLocationManager *) manager
	didFailWithError: (NSError *) error {

	addButton.enabled = NO;
}


- (void) addEvent {
	CLLocation *location = [locationManager location];
	if (!location) {
		return;
	}

	// Create and configure a new instance of the Event entity.
	Event *event = [NSEntityDescription
		insertNewObjectForEntityForName: @"Event"
		inManagedObjectContext: managedObjectContext
	];

	CLLocationCoordinate2D coordinate = [location coordinate];
	[event setLatitude: [NSNumber numberWithDouble: coordinate.latitude]];
	[event setLongitude: [NSNumber numberWithDouble: coordinate.longitude]];
	[event setCreationDate: [NSDate date]];
	
	NSError *error = nil;
	if (![managedObjectContext save: &error]) {
		// Handle the error.
		NSLog(@"save: failed: %@", error);
		[error release];
	}
	
	[eventsArray insertObject: event atIndex: 0];

	//Insert a new row at the top of the table view.
	NSIndexPath *indexPath =
		[NSIndexPath indexPathForRow: 0 inSection: 0];
	NSArray *indexPaths = [NSArray arrayWithObject: indexPath];

	[self.tableView insertRowsAtIndexPaths: indexPaths
		withRowAnimation: UITableViewRowAnimationFade];

	[self.tableView scrollToRowAtIndexPath: indexPath
		atScrollPosition: UITableViewScrollPositionTop
		animated: YES
	];
}

@end
